﻿;******************************************************************************
;	A.I.VOICE in the Tray Backend
;
;	目的
;		ウィンドウ操作にのみ管理者権限を持たせる。
;******************************************************************************

;------------------------------------------------------------------------------
; Autoexec
;------------------------------------------------------------------------------
	;環境設定
	#NoEnv							;環境変数無視
	#NoTrayIcon 						;トレイアイコン非表示起動
	#SingleInstance Force					;多重起動禁止
	SetWorkingDir		, %A_ScriptDir%			;作業ディレクトリ変更
	DetectHiddenWindows	, On				;非表示ウィンドウを検出
	OnExit	, Sub_EndProcessing				;終了処理指定


	;定数用変数
	;ソフト情報設定
	APP_TILTE	:= "A.I.VOICE in the Tray Backend"	;タイトル
	APP_VER		:= "20230901"				;バージョン
	APP_ICON	:= "Icon\A.I.VOICE in the Tray.ico"	;メインアイコン(バックアップ用)

	;Iniファイル関係
	INI_FILENAME	:= "Config.ini"				;ファイル名
	INI_SECTION	:= "AIVoice"				;セクション名

	;エディタ名（メニュー項目名用）
	EDITOR_NAME	:= "A.I.VOICE Editor"

	;フロントエンドのhwnd
	hwnd_frontend	= %1%

	;UIイニシャライズ
	TreyIcon_Init()						;トレイアイコンメニュー初期化
	SetTreyIcon()						;アイコン設定
	Gui_Init()						;GUIウィンドウ初期化

	;フロントエンドが存在しない（≒単体起動）なら終了。
	If !WinExist( "ahk_id " . hwnd_frontend )
		ExitApp

	;フロントエンド通信用メッセージハンドラ
	OnMessage( 0x8001, "RemoteSwitch")			;ウィンドウ操作処理
	OnMessage( 0x8002, "UpdateConfig")			;設定ファイル再読み込み
	OnMessage( 0x8003, "OpenGUI")				;設定ファイル再読み込み

	;非特権プロセス(フロントエンド)からのウィンドウメッセージ受信許可
	DllCall("ChangeWindowMessageFilterEx"
			, Ptr	, app_hwnd			;hwnd
			, Uint	, 0x8001			;message
			, Uint	, 1				;action	= MSGFLT_ALLOW
			, Ptr	, 0 )				;pChangeFilterStruct

	DllCall("ChangeWindowMessageFilterEx"
			, Ptr	, app_hwnd			;hwnd
			, Uint	, 0x8002			;message
			, Uint	, 1				;action	= MSGFLT_ALLOW
			, Ptr	, 0 )				;pChangeFilterStruct

	DllCall("ChangeWindowMessageFilterEx"
			, Ptr	, app_hwnd			;hwnd
			, Uint	, 0x8003			;message
			, Uint	, 1				;action	= MSGFLT_ALLOW
			, Ptr	, 0 )				;pChangeFilterStruct

	;フロントエンドに自身のhWndを伝える。
	SendMessage	, 0x8000				;WM_APP
			, 0					;wParam
			, %app_hwnd%				;lParam
			,					;Control
			, ahk_id %hwnd_frontend%

	;ウィンドウ状態取得
	Gosub , Sub_FindWindowAll

	;タイマセット
	SetTimer, Sub_FindWindowAll, 2000

	;トレイアイコン表示
	;Menu, Tray, Icon	;デバッグ時用

	Return


;------------------------------------------------------------------------------
; 終了処理
;------------------------------------------------------------------------------
Sub_EndProcessing:
	Gosub	, Sub_ShowWindowAll	;ウィンドウを表示状態に復帰
	ExitApp				;本終了
	Return


;******************************************************************************
; フロントエンド通信用メッセージハンドラ
;******************************************************************************
;------------------------------------------------------------------------------
; フロントエンドから送られて来るメッセージに従い、ウィンドウを操作する。
;------------------------------------------------------------------------------
;	msg		Ox8001
;	wParam		アプリケーション番号（0オリジン）
;			0:Editor
;			1:PotatoVoiceHub
;			2:VTPRH
;
;	lParam		0:格納	1:復帰
;
;	返り値		0:操作を実行した(結果は不問)
;			1:引数指定エラー
;------------------------------------------------------------------------------
RemoteSwitch(wParam,lParam,msg,hwnd){
	Critical	;新規は後回し

	If ( wParam > 2 )
	|| ( lParam > 1 )
		Return 1

	;操作対象
	If ( wParam == 0 )
		target_name	:= "Editor"

	If ( wParam == 1 )
		target_name	:= "PotatoVoiceHub"

	If ( wParam == 2 )
		target_name	:= "VTPRH"

	;操作モード
	If ( lParam )
		ShowWindow(target_name)
	Else
		HideWindow(target_name)

	Return 0
}


;------------------------------------------------------------------------------
; 設定ファイル再読み込み
;------------------------------------------------------------------------------
;	msg		Ox8002
;	wParam		使用しない
;	lParam		使用しない
;	返り値		0固定
;------------------------------------------------------------------------------
UpdateConfig(wParam,lParam,msg,hwnd){
	;トレイアイコン再設定
	SetTreyIcon()

	Return 0
}


;------------------------------------------------------------------------------
; GUIオープン
;------------------------------------------------------------------------------
;	msg		Ox8003
;	wParam		使用しない
;	lParam		使用しない
;	返り値		0固定
;------------------------------------------------------------------------------
OpenGUI(wParam,lParam,msg,hwnd){
	Gui, Show

	Return 0
}


;******************************************************************************
; GUIウィンドウ周り
;******************************************************************************
;------------------------------------------------------------------------------
; GUIウィンドウ初期化
;------------------------------------------------------------------------------
Gui_Init(){
	;変数はグローバル扱い(コントロール識別用)
	global

	;配置パラメータ(個別でローカル変数とする)
	local GUI_CONTROL_PITCH_X		:= 5
	local GUI_CONTROL_PITCH_Y		:= 2

	local GUI_CONTROL_WIDTH_LABEL		:= 180
	local GUI_CONTROL_WIDTH_HWND		:= 70
	local GUI_CONTROL_WIDTH_STATUS		:= 40
	local GUI_CONTROL_WIDTH_STYLE		:= 80
	local GUI_CONTROL_WIDTH_BUTTON		:= 50
	local GUI_CONTROL_HIGHT			:= 20


	;ラベル
	Gui, Add, Text	, xm ym w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, Application

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT%
			, hWnd

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, Hidden

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, MinMax

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT%
			, Style

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT%
			, ExStyle

	Gui, Add, Text	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT%
			, Tray


	;Editor
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, %EDITOR_NAME%

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_Editor_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Find_Editor gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Hide_Editor gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Show_Editor gSub_ShowWindowButton
			, Show


	;PotatoVoiceHub
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, PotatoVoiceHub

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_PotatoVoiceHub_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Find_PotatoVoiceHub gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Hide_PotatoVoiceHub gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Show_PotatoVoiceHub gSub_ShowWindowButton
			, Show


	;Voiceroid Talk Plus Receiver Hub
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, Voiceroid Talk Plus Receiver Hub

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_hWnd
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Hidden
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_MinMax
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Style
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STYLE% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_ExStyle
			, 0

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_STATUS% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_VTPRH_Tray
			, 0

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Find_VTPRH gSub_FindWindowButton
			, Find

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Hide_VTPRH gSub_HideWindowButton
			, Hide

	Gui, Add, Button, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_BUTTON% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlButton_Show_VTPRH gSub_ShowWindowButton
			, Show


	;App
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, App

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_App_hWnd
			, 0


	;FrontEnd
	Gui, Add, Text	, xm y+%GUI_CONTROL_PITCH_Y% w%GUI_CONTROL_WIDTH_LABEL% h%GUI_CONTROL_HIGHT%
			, FrontEnd

	Gui, Add, Edit	, x+%GUI_CONTROL_PITCH_X% yp w%GUI_CONTROL_WIDTH_HWND% h%GUI_CONTROL_HIGHT% ReadOnly vCtrlEdit_FrontEnd_hWnd
			, %hwnd_frontend%


	;ツールウィンドウ
	Gui,+ToolWindow

	;自身のhWndを取得
	Gui, +LastFound
	WinGet, app_hwnd, ID
	GuiControl	, Text	, CtrlEdit_App_hWnd	, %app_hwnd%

	;ウィンドウ仮表示
	Gui, Show, Hide, ウィンドウ認識状態 - %APP_TILTE%

	Return
}


;------------------------------------------------------------------------------
; GUIウィンドウからの操作
;------------------------------------------------------------------------------
;グローバル変数使いたくないので関数化
FxStringReplace(inputvar, searchtext, replacetext="", replaceall=""){
	StringReplace	, outputvar, inputvar, %searchtext%, %replacetext%, %replaceall%
	Return outputvar
}

;ウィンドウ検索ボタン
Sub_FindWindowButton:
	FindWindow(FxStringReplace( A_GuiControl, "CtrlButton_Find_"))
	Return


;ウィンドウ格納ボタン
Sub_HideWindowButton:
	HideWindow(FxStringReplace( A_GuiControl, "CtrlButton_Hide_"))
	Return


;ウィンドウ復帰ボタン
Sub_ShowWindowButton:
	ShowWindow(FxStringReplace( A_GuiControl, "CtrlButton_Show_"))
	Return



;******************************************************************************
; トレイアイコンメニュー関連
;******************************************************************************
;------------------------------------------------------------------------------
; トレイアイコンメニュー 初期化
;------------------------------------------------------------------------------
TreyIcon_Init(){
	;グローバル変数アクセス権
	global APP_TILTE, EDITOR_NAME

	;タイトル設定
	Menu, Tray, Tip, %APP_TILTE%

	;メニュー項目
	Menu, Tray, NoStandard		;標準項目抹消

	Menu, Tray, Add 	, %EDITOR_NAME%を格納			, Sub_EditorSwitch
	Menu, Tray, Add 	, PotatoVoiceHubを格納			, Sub_PotatoVoiceHubSwitch
	Menu, Tray, Add 	, VoiceroidTalkPlusReceiverHubを格納	, Sub_VTPRHSwitch
	Menu, Tray, Add

	Menu, Tray, Add 	, すべて表示				, Sub_ShowWindowAll
	Menu, Tray, Add 	, すべて格納				, Sub_HideWindowAll
	Menu, Tray, Add

	;Menu, Tray, Add 	, 全ウィンドウ再検索			, Sub_FindWindowAll
	Menu, Tray, Add 	, ウィンドウ認識状態			, Sub_Gui_Show
	Menu, Tray, Add 	, ツールを終了				, Sub_End

	Return
}


;------------------------------------------------------------------------------
; トレイアイコンメニュー デフォルト切替
;------------------------------------------------------------------------------
TreyIcon_SetDefault(){
	;グローバル変数アクセス権
	global EDITOR_NAME

	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. "Editor"	. "_hWnd"	;hWnd

	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%


	;Editorを認識しているか？
	If ( hwnd )
		Menu, Tray, Default	, %EDITOR_NAME%を格納
	Else
		Menu, Tray, NoDefault

	Return
}


;------------------------------------------------------------------------------
; トレイアイコンメニューからの操作
;------------------------------------------------------------------------------
;Editorの格納状態切替
Sub_EditorSwitch:
	SwitchWindow("Editor")
	Return

;PotatoVoiceHubの格納状態切替
Sub_PotatoVoiceHubSwitch:
	SwitchWindow("PotatoVoiceHub")
	Return

;VoiceroidTalkPlusReceiverHubの格納状態切替
Sub_VTPRHSwitch:
	SwitchWindow("VTPRH")
	Return

;------------------------------------------------------------------------------
;全ウィンドウ表示
Sub_ShowWindowAll:
	ShowWindow("PotatoVoiceHub")	;PotatoVoiceHub
	ShowWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	ShowWindow("Editor")		;Editor
	Return

;全ウィンドウを隠す
Sub_HideWindowAll:
	HideWindow("Editor")		;Editor
	HideWindow("PotatoVoiceHub")	;PotatoVoiceHub
	HideWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	Return

;------------------------------------------------------------------------------
;全ウィンドウ再検索
Sub_FindWindowAll:
	FindWindow("Editor")		;Editor
	FindWindow("PotatoVoiceHub")	;PotatoVoiceHub
	FindWindow("VTPRH")		;Voiceroid Talk Plus Receiver Hub
	TreyIcon_SetDefault()		;タスクトレイアイコンデフォルト切替

	;フロントエンド実行中チェック
	If ( !WinExist("ahk_id " . hwnd_frontend))	;追従して終了する。
		ExitApp

	Return

;ウィンドウ認識状態表示
Sub_Gui_Show:
	Gui, Show
	Return

;ツールを終了
Sub_End:
	ExitApp		;終了処理へ移行
	Return



;******************************************************************************
; トレイアイコン関連
;******************************************************************************
;------------------------------------------------------------------------------
; トレイアイコン設定
;------------------------------------------------------------------------------
SetTreyIcon(){
	;グローバル変数アクセス権
	global INI_FILENAME, INI_SECTION, APP_ICON

	;設定ファイル読込
	IniRead	, tray_icon
		, %INI_FILENAME%
		, %INI_SECTION%
		, Icon

	;存在チェック
	If ( FileExist( tray_icon ) )
		Menu, Tray, Icon, %tray_icon%
	Else
		Menu, Tray, Icon, %APP_ICON%

	Return
}


;******************************************************************************
; その他関数
;******************************************************************************
;------------------------------------------------------------------------------
; 共通処理部
; 個別・一括操作ルーチンから呼ばれるコア部分
;------------------------------------------------------------------------------
;ウィンドウ検索
FindWindow(target_name){
	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	If ( target_name == "Editor" )		;Editor
	{
		Process, Exist, AIVoiceEditor.exe
		hwnd := WinExist( "A.I.VOICE Editor" . " ahk_pid " . ErrorLevel )
	}

	If ( target_name == "PotatoVoiceHub" )	;PotatoVoiceHub
	{
		Process, Exist, PotatoVoiceHub.exe
		hwnd := WinExist( "PotatoVoiceHub" . " ahk_pid " . ErrorLevel )
	}

	If ( target_name == "VTPRH" )		;Voiceroid Talk Plus Receiver Hub
	{
		Process, Exist, VoiceroidTalkPlusReceiverHub.exe
		hwnd := WinExist( "Voiceroid Talk Plus Receiver" . " ahk_pid " . ErrorLevel )
	}


	;書き込む
	GuiControl	, Text	, %ctrl_hwnd%	, %hwnd%


	;そのままウィンドウステータス取得
	GetWindowStatus(target_name)

	;状態補正
	FixWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウステータス取得
;------------------------------------------------------------------------------
GetWindowStatus(target_name){
	;グローバル変数アクセス権
	global EDITOR_NAME

	;変数初期化
	hwnd		:= 0
	hidden		:= 0
	minmax		:= 0
	style		:= 0
	exstyle		:= 0
	tray		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd
	ctrl_hidden	:= "CtrlEdit_"	. target_name	. "_Hidden"	;非表示状態
	ctrl_minmax 	:= "CtrlEdit_"	. target_name	. "_MinMax"	;最大化・最小化状態
	ctrl_style 	:= "CtrlEdit_"	. target_name	. "_Style"	;ウィンドウスタイル
	ctrl_exstyle 	:= "CtrlEdit_"	. target_name	. "_ExStyle"	;ウィンドウ拡張スタイル
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウステータス取得
	If ( hwnd )
	{
		WinGet, minmax	, MinMax	, ahk_id %hwnd%
		WinGet, style	, Style		, ahk_id %hwnd%
		WinGet, exstyle	, ExStyle	, ahk_id %hwnd%

		;非表示状態チェック
		DetectHiddenWindows, Off				;非表示ウィンドウ検出OFF

		If !WinExist( "ahk_id " . hwnd )
			hidden := 1

		DetectHiddenWindows, On					;非表示ウィンドウ検出ON
	}


	;トレイ化状態判別
	;(トレイ化とみなす条件はアプリケーションによって異なる為判別ロジックも異なる。)
	;Editor
	If ( target_name == "Editor" )
	{
		;格納状態判別条件＝最小化＆ツールウィンドウ状態

		;ツールウィンドウか判別
		Transform	, exstyle_toold
				, BitAnd
				, exstyle
				, 0x00000080

		;判定
		If ( minmax == -1 )
		&& ( exstyle_toold )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, %EDITOR_NAME%を格納
		Else
			Menu, Tray, UnCheck, %EDITOR_NAME%を格納
	}

	;PotatoVoiceHub
	If ( target_name == "PotatoVoiceHub" )
	{
		;格納状態判別条件＝最小化＆非表示
		If ( minmax == "-1" )
		&& ( hidden )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, PotatoVoiceHubを格納
		Else
			Menu, Tray, UnCheck, PotatoVoiceHubを格納

	}

	;Voiceroid Talk Plus Receiver Hub
	If ( target_name == "VTPRH" )
	{
		;格納状態判別条件＝非表示
		If ( hidden )
			tray	:= 1

		;トレイアイコンメニューチェック状態切替
		If ( tray )
			Menu, Tray, Check, VoiceroidTalkPlusReceiverHubを格納
		Else
			Menu, Tray, UnCheck, VoiceroidTalkPlusReceiverHubを格納
	}


	;結果を書き込む
	GuiControl	, Text	, %ctrl_hidden%		, %hidden%
	GuiControl	, Text	, %ctrl_minmax%		, %minmax%
	GuiControl	, Text	, %ctrl_style%		, %style%
	GuiControl	, Text	, %ctrl_exstyle%	, %exstyle%
	GuiControl	, Text	, %ctrl_tray%		, %tray%

	Return
}


;------------------------------------------------------------------------------
; 状態補正
; 格納状態で新しく起動したときなどのウィンドウ状態の自動修正や
; 最小化時に自動でトレイアイコン化するなど。
;------------------------------------------------------------------------------
FixWindowStatus(target_name){
	;変数初期化
	hwnd		:= 0
	hidden		:= 0
	minmax		:= 0
	style		:= 0
	exstyle		:= 0
	tray		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd
	ctrl_hidden	:= "CtrlEdit_"	. target_name	. "_Hidden"	;非表示状態
	ctrl_minmax 	:= "CtrlEdit_"	. target_name	. "_MinMax"	;最大化・最小化状態
	ctrl_style 	:= "CtrlEdit_"	. target_name	. "_Style"	;ウィンドウスタイル
	ctrl_exstyle 	:= "CtrlEdit_"	. target_name	. "_ExStyle"	;ウィンドウ拡張スタイル
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウステータス取得
	If ( hwnd )
	{
		GuiControlGet	, hidden	,	, %ctrl_hidden%
		GuiControlGet	, minmax	,	, %ctrl_minmax%
		GuiControlGet	, style		,	, %ctrl_style%
		GuiControlGet	, exstyle	,	, %ctrl_exstyle%
		GuiControlGet	, tray		,	, %ctrl_tray%


		;Editorの復帰漏れ(≒ミス)対策
		If ( target_name == "Editor" )
		{
			;ツールウィンドウか判別
			Transform	, exstyle_toold
					, BitAnd
					, exstyle
					, 0x00000080

			;判定
			If ( minmax != -1 )
			&& ( exstyle_toold )
				ShowWindow(target_name)
		}


		;補助アプリは最小化で非格納なら格納させる。
		If ( target_name == "PotatoVoiceHub" )
		|| ( target_name == "VTPRH" )
		{
			If ( minmax == -1 )
			&& ( tray == 0 )
				HideWindow(target_name)
		}
	}
}


;------------------------------------------------------------------------------
; ウィンドウ格納
;------------------------------------------------------------------------------
HideWindow(target_name){
	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウを隠す
	If ( hwnd )
	{
		;アプリ個別対応
		If ( target_name == "Editor" )
		{
			WinMinimize , ahk_id %hwnd%		;最小化

			;ウィンドウスタイル変更
			WinSet	, ExStyle
				, -0x00040000			;WS_EX_APPWINDOW
				, ahk_id %hwnd%

			WinSet	, ExStyle
				, +0x00000080			;WS_EX_TOOLWINDOW
				, ahk_id %hwnd%
		}

		If ( target_name == "PotatoVoiceHub" )
		{
			WinMinimize , ahk_id %hwnd%		;最小化
			WinHide	, ahk_id %hwnd%			;ウィンドウ非表示化
		}

		If ( target_name == "VTPRH" )
		{
			WinRestore , ahk_id %hwnd%		;ウィンドウ復帰
			WinHide	, ahk_id %hwnd%			;ウィンドウ非表示化
		}
	}

	;ステータス最新化
	GetWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウ格納から復帰
;------------------------------------------------------------------------------
ShowWindow(target_name){
	;変数初期化
	hwnd		:= 0

	;コントロール名
	ctrl_hwnd 	:= "CtrlEdit_"	. target_name	. "_hWnd"	;hWnd


	;hWnd取得
	GuiControlGet	, hwnd	,	, %ctrl_hwnd%

	;ウィンドウを表示
	If ( hwnd )
	{
		;アプリ個別対応
		If ( target_name == "Editor" )
		{
			;ウィンドウスタイル復旧
			WinSet	, ExStyle
				, +0x00040000			;WS_EX_APPWINDOW
				, ahk_id %hwnd%

			WinSet	, ExStyle
				, -0x00000080			;WS_EX_TOOLWINDOW
				, ahk_id %hwnd%

			WinRestore , ahk_id %hwnd%		;ウィンドウ復帰
		}

		If ( target_name == "PotatoVoiceHub" )
		|| ( target_name == "VTPRH" )
		{
			WinShow	, ahk_id %hwnd%			;ウィンドウ再表示
			WinRestore , ahk_id %hwnd%		;ウィンドウ復帰
		}
	}

	;ステータス最新化
	GetWindowStatus(target_name)

	Return
}


;------------------------------------------------------------------------------
; ウィンドウ格納状態切替
;------------------------------------------------------------------------------
SwitchWindow(target_name){
	;変数初期化
	tray		:= 0

	;コントロール名
	ctrl_tray 	:= "CtrlEdit_"	. target_name	. "_Tray"	;格納状態


	;ステータス最新化
	GetWindowStatus(target_name)


	;格納状態取得
	GuiControlGet	, tray	,	, %ctrl_tray%

	;格納されているか？
	If ( tray )
		ShowWindow(target_name)		;復帰
	Else
		HideWindow(target_name)		;格納

	Return
}
